/**
 * Ported from the Java example program in
 * Head First Design Patterns, 1st edition
 * by Eric Freeman, Bert Bates, and Elizabeth Robson
 * O'Reilly, 2004
 * ISBN 978-0596007126
 *
 * Ported to C++ by:
 * Ronald Mak, Nov. 2019
 * Department of Computer Engineering
 * San Jose State University
 */
#ifndef GUMBALLMACHINE_H_
#define GUMBALLMACHINE_H_

#include <iostream>

#include "HasQuarterState.h"
#include "NoQuarterState.h"
#include "SoldOutState.h"
#include "SoldState.h"
#include "WinnerState.h"

using namespace std;

class GumballMachine
{
private:
    State *has_quarter_state;
    State *no_quarter_state;
    State *sold_out_state;
    State *sold_state;
    State *winner_state;

public:
    State *state;  // the current state
    int count;     // how many gumballs left

    GumballMachine(int count)
        : has_quarter_state(new HasQuarterState(this)),
          no_quarter_state(new NoQuarterState(this)),
          sold_out_state(new SoldOutState(this)),
          sold_state(new SoldState(this)),
          winner_state(new WinnerState(this)),
          count(count)
    {
        state = count > 0 ? no_quarter_state
                          : sold_out_state;
    }

    virtual ~GumballMachine()
    {
        delete has_quarter_state;
        delete no_quarter_state;
        delete sold_out_state;
        delete sold_state;
        delete winner_state;
    }

    int    get_count() const { return count; }
    State *get_state() const { return state; }

    void set_state(State *st) { state = st; }

    State *get_has_quarter_state() const { return has_quarter_state; }
    State *get_no_quarter_state()  const { return no_quarter_state; }
    State *get_sold_out_state()    const { return sold_out_state; }
    State *get_sold_state()        const { return sold_state; }
    State *get_winner_state()      const { return winner_state; }

    void insert_quarter() { state->insert_quarter(); }
    void eject_quarter()  { state->eject_quarter(); }

    void turn_crank()
    {
        state->turn_crank();
        state->dispense();
    }

    void release_ball();

    friend ostream& operator <<(ostream& os, const GumballMachine& gumball);
};

#endif /* GUMBALLMACHINE_H_ */
